@@ -564,6 +564,7 @@ class ClauseDB(LogicProgram):
                 return filename + '.pl'
             elif os.path.exists(filename + '.py'):
                 return filename + '.py'
+        return filename
 
     def create_function(self, functor, arity):
         """Create a Python function that can be used to query a specific predicate on this database.
@@ -605,6 +606,8 @@ class ClauseDB(LogicProgram):
             self.line_info.append(program.line_info[0])
             # engine._process_directives(database)
             return self.add_all(program)
+        else:
+            return None, None
 
     def add_all(self, program):
         module_name = None
