@@ -27,7 +27,7 @@ from __future__ import print_function
 
 from .util import Timer, mktempfile
 from .formula import LogicFormula, atom, LogicNNF
-from .evaluator import EvaluatableDSP, Evaluator, FormulaEvaluatorNSP, FormulaEvaluator, SemiringLogProbability, SemiringProbability
+from .evaluator import EvaluatableDSP, Evaluator, FormulaEvaluatorNSP, SemiringLogProbability, SemiringProbability
 from .errors import InconsistentEvidenceError
 
 
@@ -39,7 +39,7 @@ class DD(LogicFormula, EvaluatableDSP):
 
         self.inode_manager = None
 
-        self.atom2var = {}
+        self.atom2var = {}  # index to node
         self.var2atom = {}
 
         # self._constraint_dd = None
@@ -67,31 +67,29 @@ class DD(LogicFormula, EvaluatableDSP):
         :param index: index of node to retrieve
         :return: internal node corresponding to the given index
         """
-        negate = False
+        mgr = self.get_manager()  # type: DDManager
         if self.is_false(index):
-            return self.get_manager().false()
+            return mgr.false()
         elif self.is_true(index):
-            return self.get_manager().true()
-        elif index < 0:
-            index = -index
-            negate = True
+            return mgr.true()
+
+        negate = index < 0
+        index = abs(index)
         node = self.get_node(index)
+
         if type(node).__name__ == 'atom':
-            result = self.get_manager().literal(self.atom2var[index])
+            result = mgr.literal(self.atom2var[index])
         else:
             # Extend list
-            while len(self.get_manager().nodes) < index:
-                self.get_manager().nodes.append(None)
-            if self.get_manager().nodes[index - 1] is None:
-                self.get_manager().nodes[index - 1] = self._create_inode(node)
-            result = self.get_manager().nodes[index - 1]
-        if negate:
-            new_sdd = self.get_manager().negate(result)
-            return new_sdd
-        else:
-            return result
+            while len(mgr.nodes) < index:
+                mgr.nodes.append(None)
+            result = mgr.nodes[index - 1]
+            if result is None:
+                result = self._create_inode(node)
+                mgr.nodes[index - 1] = result
+        return mgr.negate(result) if negate else result
 
-    def _create_inode(self, node):
+    def _create_inode(self, node):  # TODO: Recursion is slow in python. Change build_dd to not use this and get_inode?
         if type(node).__name__ == 'conj':
             return self.get_manager().conjoin(*[self.get_inode(c) for c in node.children])
         else:
@@ -124,7 +122,7 @@ class DD(LogicFormula, EvaluatableDSP):
     def build_dd(self):
         """Build the internal representation of the formula."""
         required_nodes = set([abs(n) for q, n, l in self.labeled() if self.is_probabilistic(n)])
-        required_nodes |= set([abs(n) for q, n, l in self.labeled() if self.is_probabilistic(n)])
+        required_nodes |= set([abs(n) for q, n, l in self.labeled() if self.is_probabilistic(n)])  # TODO self.evidence_all() ipv self.labeled() ? see forward.py
 
         for n in required_nodes:
             self.get_inode(n)
@@ -144,11 +142,15 @@ class DD(LogicFormula, EvaluatableDSP):
 
     def to_dot(self, *args, **kwargs):
         if kwargs.get('use_internal'):
+            dot_text = ["digraph {\n", "overlap=false\n"]  # TODO Support multiple nodes in write_to_dot
             for qn, qi in self.queries():
                 filename = mktempfile('.dot')
                 self.get_manager().write_to_dot(self.get_inode(qi), filename)
                 with open(filename) as f:
-                    return f.read()
+                    dot_text += f.readlines()[4:-1]
+
+            dot_text.append("}")
+            return "".join(dot_text)
         else:
             return self.to_formula().to_dot(*args, **kwargs)
 
@@ -159,7 +161,7 @@ class DDManager(object):
     """
 
     def __init__(self):
-        self.nodes = []
+        self.nodes = []  # Stores inodes (only conjoin and disjoin, the remaining atoms are in formula.atom2var)
         self.constraint_dd = None
 
     def set_node(self, index, node):
@@ -384,7 +386,6 @@ class DDEvaluator(Evaluator):
     :param semiring:
     :param weights:
     :return:
-
     """
 
     def __init__(self, formula, semiring, weights=None, **kwargs):
@@ -408,21 +409,22 @@ class DDEvaluator(Evaluator):
             av = self.formula.atom2var.get(atom)
             if av is not None:
                 self.weights[av] = weight
+            elif atom == 0:
+                self.weights[0] = weight
 
         if with_evidence:
             for ev in self.evidence():
-                if ev in self.formula.atom2var:
+                if abs(ev) in self.formula.atom2var:
                     # Only for atoms
-                    self.set_evidence(self.formula.atom2var[ev], ev > 0)
+                    self.set_evidence(self.formula.atom2var[abs(ev)], ev > 0)
 
     def propagate(self):
         self._initialize()
         if isinstance(self.semiring, SemiringLogProbability) or isinstance(self.semiring, SemiringProbability):
             self.normalization = self._get_manager().wmc_true(self.weights, self.semiring)
-
         else:
             self.normalization = None
-        self.evaluate_evidence()
+        self.evaluate_evidence(recompute=True)
 
     def _evaluate_evidence(self, recompute=False):
         if self._evidence_weight is None or recompute:
@@ -432,13 +434,23 @@ class DDEvaluator(Evaluator):
 
             if isinstance(self.semiring, SemiringLogProbability) or isinstance(self.semiring, SemiringProbability):
                 result = self._get_manager().wmc(self.evidence_inode, self.weights, self.semiring)
-                if result == self.semiring.zero():
+                if self.semiring.is_zero(result):
                     raise InconsistentEvidenceError(context=' during compilation')
                 self._evidence_weight = self.semiring.normalize(result, self.normalization)
             else:
                 formula = LogicNNF()
                 i = self.formula._to_formula(formula, self.evidence_inode)
-                self._evidence_weight = formula.evaluate(index=i, semiring=self.semiring)
+                if i is None:  # = Node(False)
+                    # evaluate(index=i=None,..) evaluates the LABEL.QUERY's of formula.
+                    formula.add_name(name='tempFalse', key=None, label=formula.LABEL_QUERY)
+                    result = formula.evaluate(index=i, semiring=self.semiring)
+                    result = result['tempFalse']
+                else:
+                    result = formula.evaluate(index=i, semiring=self.semiring)
+
+                if self.semiring.is_zero(result):
+                    raise InconsistentEvidenceError(context=' during compilation')
+                self._evidence_weight = result
 
         return self._evidence_weight
 
@@ -455,7 +467,7 @@ class DDEvaluator(Evaluator):
         # Trivial case: node is deterministically True or False
         if node == self.formula.TRUE:
             result = self.semiring.one()
-        elif node is self.formula.FALSE:
+        elif node == self.formula.FALSE:
             result = self.semiring.zero()
         else:
             query_def_inode = self.formula.get_inode(node)
@@ -484,16 +496,21 @@ class DDEvaluator(Evaluator):
             evidence_inode = self.evidence_inode
             query_sdd = self._get_manager().conjoin(query_def_inode, evidence_inode)
 
-
-
             formula = LogicNNF()
             i = self.formula._to_formula(formula, query_sdd)
-            result = formula.evaluate(index=i, semiring=self.semiring)
+
+            if i is None:  # = SddNode(False)
+                # evaluate(index=i=None,..) evaluates the LABEL.QUERY's of formula.
+                name = self.formula.get_name(node)
+                formula.add_name(name=name, key=None, label=formula.LABEL_QUERY)
+                result = formula.evaluate(index=i, semiring=self.semiring)
+                # index=None such that this if i is None is unnecessary. (same structure at _evaluate_evidence(..))
+                result = result[name]
+            else:
+                result = formula.evaluate(index=i, semiring=self.semiring)
 
             self._get_manager().deref(query_sdd)
 
-
-
             # TODO only normalize when there are evidence or constraints.
 #            result = self.semiring.normalize(result, self.normalization)
             result = self.semiring.normalize(result, self._evidence_weight)
@@ -507,26 +524,23 @@ class DDEvaluator(Evaluator):
 
         inode = self.evidence_inode
 
-        result = self.semiring.result(
-            self._get_manager().wmc_literal(
-                inode, self.weights, self.semiring, self.formula.atom2var[node]), self.formula)
+        literal_result = self._get_manager().wmc_literal(inode, self.weights, self.semiring,
+                                                         self.formula.atom2var[node])
+        result = self.semiring.result(literal_result, self.formula)
         return result
 
     def set_evidence(self, index, value):
-        pos = self.semiring.one()
-        neg = self.semiring.zero()
+        curr_pos_weight, curr_neg_weight = self.weights.get(index)
+        pos, neg = self.semiring.to_evidence(curr_pos_weight, curr_neg_weight, sign=value)
 
-        current_weight = self.weights.get(index)
-        if value:
-            if current_weight and self.semiring.is_zero(current_weight[0]):
-                raise InconsistentEvidenceError(self._deref_node(index))
-            self.set_weight(index, pos, neg)
-        else:
-            if current_weight and self.semiring.is_one(current_weight[0]):
-                raise InconsistentEvidenceError(self._deref_node(index))
-            self.set_weight(index, neg, pos)
+        if (value and self.semiring.is_zero(curr_pos_weight)) or \
+                (not value and self.semiring.is_zero(curr_neg_weight)):
+            raise InconsistentEvidenceError(self._deref_node(index))
+
+        self.set_weight(index, pos, neg)
 
     def set_weight(self, index, pos, neg):
+        # index = index of atom in weights, so atom2var[key] = index
         self.weights[index] = (pos, neg)
 
     def _deref_node(self, index):
@@ -547,27 +561,23 @@ def build_dd(source, destination, **kwdargs):
     :param kwdargs: extra arguments
     :return: destination
     """
-
     with Timer('Compiling %s' % destination.__class__.__name__):
 
         # TODO maintain a translation table
         for i, n, t in source:
             if t == 'atom':
-                j = destination.add_atom(n.identifier, n.probability, n.group, source.get_name(i))
+                j = destination.add_atom(n.identifier, n.probability, n.group, name=source.get_name(i), cr_extra=False)
             elif t == 'conj':
-                j = destination.add_and(n.children, source.get_name(i))
+                j = destination.add_and(n.children, name=n.name)
             elif t == 'disj':
-                j = destination.add_or(n.children, source.get_name(i))
+                j = destination.add_or(n.children, name=n.name)
             else:
                 raise TypeError('Unknown node type')
-            assert i == j
+            # assert i == j  # Does not hold with constraints. See if-comment.
 
         for name, node, label in source.get_names_with_label():
             destination.add_name(name, node, label)
 
-        for c in source.constraints():
-            if c.is_nontrivial():
-                destination.add_constraint(c)
         destination.build_dd()
 
     return destination
