@@ -196,6 +196,7 @@ def add_standard_builtins(engine, b=None, s=None, sp=None):
 
     for i in range(1, 10):
         engine.add_builtin('writenl', i, b(_builtin_writenl))
+        engine.add_builtin('writeln', i, b(_builtin_writenl))
 
     for i in range(1, 10):
         engine.add_builtin('error', i, b(_builtin_error))
@@ -1624,7 +1625,7 @@ def _use_module(scope, filename, database=None, location=None, **kwdargs):
     return True
 
 @builtin_boolean('_consult', 2)
-def _builtin_consult(scope, filename, database=None, engine=None, **kwdargs):
+def _consult(scope, filename, database=None, engine=None, **kwdargs):
     check_mode((filename,), ['a'], functor='consult', **kwdargs)
     scope = str(scope)
     if scope == 'None':
@@ -1649,6 +1650,7 @@ def _builtin_call(term, args=(), engine=None, callback=None, transform=None, con
     # Find the define node for the given query term.
     term_call = term.with_args(*(term.args + args))
 
+    from .engine_stack import get_state
     try:
         if transform is None:
             from .engine_stack import Transformations
@@ -1658,7 +1660,7 @@ def _builtin_call(term, args=(), engine=None, callback=None, transform=None, con
             n = len(term.args)
             res1 = result[:n]
             res2 = result[n:]
-            return engine.create_context([term.with_args(*res1)] + list(res2), state=result.state)
+            return engine.create_context([term.with_args(*res1)] + list(res2), state=get_state(result))
         transform.addFunction(_trans)
 
         actions = engine.call_intern(term_call, transform=transform, parent_context=context, **kwdargs)
@@ -1677,17 +1679,16 @@ def _builtin_subquery(term, prob, evidence=None, engine=None, database=None, **k
     else:
         check_mode((term, prob), ['cv'], functor='subquery')
 
-    from .sdd_formula import SDD
-
     eng = engine.__class__()
-
     target = eng.ground(database, term, label='query')
 
     if evidence:
         for ev in term2list(evidence):
             target = eng.ground(database, ev, target=target, label=target.LABEL_EVIDENCE_POS)
 
-    results = SDD.create_from(target).evaluate()
+    from . import get_evaluatable
+    kc = get_evaluatable(name=None) # TODO somehow pass evaluatable preference (name, -k)
+    results = kc.create_from(target).evaluate()
     if evidence:
         return [(t, Constant(p), evidence) for t, p in results.items()]
     else:
