@@ -472,13 +472,13 @@ class StackBasedEngine(ClauseDBEngine):
                         if act in 'rco':
                             print(obj, act, args)
                         print([(a, o, x) for a, o, x, t in actions[-10:]])
-                        if self.trace:
-                            a = sys.stdin.readline()
-                            if a.strip() == 'gp':
-                                print(target)
-                            elif a.strip() == 'l':
-                                self.trace = False
-                                self.debug = False
+                    if self.trace:
+                        a = sys.stdin.readline()
+                        if a.strip() == 'gp':
+                            print(target)
+                        elif a.strip() == 'l':
+                            self.trace = False
+                            self.debug = False
                     if cleanup:
                         self.cleanup(obj)
 
@@ -950,7 +950,7 @@ class StackBasedEngine(ClauseDBEngine):
 
             transform.addFunction(result_transform)
             return self.eval(node.child, context=new_context, parent=parent, transform=transform,
-                             current_clause=node_id, **kwdargs)
+                             current_clause=node_id, identifier=identifier, **kwdargs)
         except UnifyError:
             # Call and clause head are not unifiable, just fail (complete without results).
             return [complete(parent, identifier)]
@@ -1776,7 +1776,7 @@ class EvalDefine(EvalNode):
                         a = False
                     return a, actions
                 else:
-                    cache_key = (self.node.functor, res)
+                    cache_key = self.call  # (self.node.functor, res)
                     if not self.no_cache and cache_key in self.target._cache:
                         # Get direct
                         stored_result = self.target._cache[cache_key]
@@ -1832,7 +1832,8 @@ class EvalDefine(EvalNode):
         else:
             self.to_complete -= 1
             if self.to_complete == 0:
-                cache_key = (self.node.functor, self.context)
+                cache_key = self.call
+                # cache_key = (self.node.functor, self.context)
                 # assert (not cache_key in self.target._cache)
                 self.flushBuffer()
                 self.target._cache[cache_key] = self.results
@@ -1859,11 +1860,19 @@ class EvalDefine(EvalNode):
 
     def flushBuffer(self, cycle=False):
         def func(res, nodes):
-            cache_key = (self.node.functor, res)
+            cache_key = self.call
+#            cache_key = (self.node.functor, res)
             if not self.no_cache and cache_key in self.target._cache:
                 stored_result = self.target._cache[cache_key]
-                assert (len(stored_result) == 1)
-                node = stored_result[0][1]
+                if len(stored_result) == 1:
+                    node = stored_result[0][1]
+                else:
+                    if self.engine.label_all:
+                        name = Term(self.node.functor, *res)
+                    else:
+                        name = None
+                    node = self.target.add_or([y for x, y in stored_result], name=name)
+                # assert (len(stored_result) == 1)
                 if not self.is_root:
                     node = self.engine.propagate_evidence(self.database, self.target, self.node.functor, res, node)
             else:
