@@ -253,7 +253,12 @@ class problog_export_nondet(problog_export):
             bound = check_mode(args, list(self._extract_callmode()), funcname, **kwdargs)
             converted_args = self._convert_inputs(args)
             results = []
-            for result in func(*converted_args):
+            argspec = inspect.getargspec(func)
+            if argspec.keywords is not None:
+                func_result = func(*converted_args, **kwdargs)
+            else:
+                func_result = func(*converted_args)
+            for result in func_result:
                 if len(self.output_arguments) == 1:
                     result = [result]
 
@@ -309,4 +314,4 @@ def _call_func(func):
         else:
             return f
 
-    return wrap
\ No newline at end of file
+    return wrap
