@@ -67,8 +67,8 @@ class BaseFormula(ProbLogObject):
 
         self._constraints = []           # Constraints: list of Constraint
 
-        self._names = defaultdict(OrderedDict)  # Node names: dict(label: dict(key, Term))
-
+        #self._names = defaultdict(OrderedDict)  # Node names: dict(label: dict(key, Term))
+        self._names = defaultdict(dict)
         self._atomcount = 0
 
     @property
@@ -116,39 +116,69 @@ class BaseFormula(ProbLogObject):
     def extract_weights(self, semiring, weights=None):
         """Extracts the positive and negative weights for all atoms in the data structure.
 
+        * Atoms with weight set to neutral will get weight ``(semiring.one(), semiring.one())``.
+        * If the weights argument is given, it completely replaces the formula's weights.
+        * All constraints are applied to the weights.
+        * To specify positive and negative literal of an atom you can pass a tuple and handle it in the semiring functions pos_value and neg_value
+            (4,5)::a.
+            4 is the postive weight and 5 the negative.
+
         :param semiring: semiring that determines the interpretation of the weights
-        :param weights: dictionary of { node name : weight } that overrides the builtin weights
-        :returns: dictionary { key: (positive weight, negative weight) }
-        :rtype: dict[int, tuple[any]]
-
-        Atoms with weight set to neutral will get weight ``(semiring.one(), semiring.one())``.
-
-        If the weights argument is given, it completely replaces the formula's weights.
-
-        All constraints are applied to the weights.
+        :param weights: dictionary of { node name : weight } or { node id : weight} that overrides the builtin weights,
+            the given weights must be in external representation.
+        :type weights: dict[(Term | int), any]
+        :returns: dictionary { key: (positive weight, negative weight) } where the weights are in internal
+            representation.
+        :rtype: dict[int, tuple[any,any]]
         """
-
-        if weights is None:
-            weights = self.get_weights()
-        else:
-            oweights = dict(self.get_weights().items())
-            oweights.update({self.get_node_by_name(n): v for n, v in weights.items()})
-            weights = oweights
-
         result = {}
-        for n, w in weights.items():
-            if hasattr(self, 'get_name'):
-                name = self.get_name(n)
-            else:
-                name = n
-            if w == self.WEIGHT_NEUTRAL and type(self.WEIGHT_NEUTRAL) == type(w):
-                result[n] = semiring.one(), semiring.one()
-            elif w == False:
-                result[n] = semiring.false(name)
-            elif w is None:
-                result[n] = semiring.true(name)
-            else:
-                result[n] = semiring.pos_value(w, name), semiring.neg_value(w, name)
+        # Set given weights, has priority over program weights.
+        if weights is not None:
+            for n, w in weights.items():
+                key = n if isinstance(n, int) else self.get_node_by_name(n)
+
+                if hasattr(self, 'get_name'):
+                    name = self.get_name(key)
+                elif not isinstance(n, int):
+                    name = n
+                else:
+                    name = key
+
+                if key >= 0:
+                    if w == self.WEIGHT_NEUTRAL and type(self.WEIGHT_NEUTRAL) == type(w):
+                        result[key] = semiring.one(), semiring.one()
+                    elif w == False:
+                        result[key] = semiring.false(name)
+                    elif w is None:
+                        result[key] = semiring.true(name)
+                    else:
+                        result[key] = semiring.pos_value(w, name), semiring.neg_value(w, name)
+                else:
+                    # If key < 0 we have to reverse weight order (p,n) to (n,p)
+                    if w == self.WEIGHT_NEUTRAL and type(self.WEIGHT_NEUTRAL) == type(w):
+                        result[abs(key)] = semiring.one(), semiring.one()
+                    elif w == False:
+                        result[abs(key)] = semiring.true(name)
+                    elif w is None:
+                        result[abs(key)] = semiring.false(name)
+                    else:
+                        result[abs(key)] = semiring.neg_value(w, name), semiring.pos_value(w, name)
+
+        # Set remaining program weights
+        for key, w in self.get_weights().items():
+            if result.get(abs(key)) is None:
+                if hasattr(self, 'get_name'):
+                    name = self.get_name(key)
+                else:
+                    name = key
+                if w == self.WEIGHT_NEUTRAL and type(self.WEIGHT_NEUTRAL) == type(w):
+                    result[key] = semiring.one(), semiring.one()
+                elif w is False:
+                    result[key] = semiring.false(name)
+                elif w is None:
+                    result[key] = semiring.true(name)
+                else:
+                    result[key] = semiring.pos_value(w, name), semiring.neg_value(w, name)
 
         for c in self.constraints():
             c.update_weights(result, semiring)
@@ -164,7 +194,7 @@ class BaseFormula(ProbLogObject):
         :param name: name of the node
         :type name: Term
         :param key: key of the node
-        :type key: int | TRUE | FALSE
+        :type key: int | bool
         :param label: type of label (one of LABEL_*)
         """
         if label is None:
@@ -409,6 +439,7 @@ class LogicFormula(BaseFormula):
         self._nodes = []
         # Lookup index for 'atom' nodes, key is identifier passed to addAtom()
         self._index_atom = {}
+        self._index_next = 0
         # Lookup index for 'and' nodes, key is tuple of sorted children
         self._index_conj = {}
         # Lookup index for 'or' nodes, key is tuple of sorted children
@@ -451,19 +482,11 @@ class LogicFormula(BaseFormula):
         if self._use_string_names:
             name = str(name)
 
-        if self.is_probabilistic(key):
+        if not keep_name and self.is_probabilistic(key):
             node = self.get_node(abs(key))
-            ntype = type(node).__name__
-            if key < 0:
-                lname = -name
-            else:
-                lname = name
-            if not keep_name:
-                if ntype == 'atom':
-                    node = type(node)(*(node[:-2] + (lname, node[-1])))
-                else:
-                    node = type(node)(*(node[:-1] + (lname,)))
-                self._update(abs(key), node)
+            lname = -name if key < 0 else name
+            node = node._replace(name=lname)
+            self._update(abs(key), node)
 
         BaseFormula.add_name(self, name, key, label)
 
@@ -482,10 +505,8 @@ class LogicFormula(BaseFormula):
     def _add(self, node, key=None, reuse=True):
         """Adds a new node, or reuses an existing one.
 
-        :param node: node to add
-        :param reuse: (default True) attempt to map the new node onto an existing one based on its \
-         content
-
+        :param node: node to add (namedtuple: atom, conj or disj)
+        :param reuse: (default True) attempt to map the new node onto an existing one based on its content
         """
         if reuse:
             # Determine the node's key and lookup identifier base on node type.
@@ -507,6 +528,9 @@ class LogicFormula(BaseFormula):
                 index = len(self._nodes) + 1
                 # Add the entry to the collection
                 collection[key] = index
+                # If atom, update max index
+                if ntype == 'atom' and type(key) == int and key >= self._index_next:
+                    self._index_next = key + 1
                 # Add entry to the set of nodes
                 self._nodes.append(node)
             else:
@@ -520,6 +544,13 @@ class LogicFormula(BaseFormula):
         # Return the entry
         return index
 
+    def get_next_atom_identifier(self):
+        """
+        Get a unique identifier that can - and has not - been used to add a new atom.
+        :return: A next unique identifier to use when adding new atoms (self.add_atom(identifier=..))
+        """
+        return self._index_next
+
     def _update(self, key, value):
         """Replace the node with the given content.
 
@@ -531,14 +562,21 @@ class LogicFormula(BaseFormula):
         assert(key > 0)
         self._nodes[key - 1] = value
 
-    def _add_constraint_me(self, group, node):
+    def _add_constraint_me(self, group, node, cr_extra=True):
+        """
+
+        :param group:
+        :param node:
+        :param cr_extra: When required, create an additional extra_node atom for the group.
+        :return:
+        """
         if group is None:
             return node
         constraint = self._constraints_me.get(group)
         if constraint is None:
             constraint = ConstraintAD(group)
             self._constraints_me[group] = constraint
-        node = constraint.add(node, self)
+        node = constraint.add(node, self, cr_extra=cr_extra)
         return node
 
     def force_weights(self):
@@ -548,7 +586,7 @@ class LogicFormula(BaseFormula):
                 w = weights.get(i, n.probability)
                 self._nodes[i - 1] = atom(n.identifier, w, n.group, n.name, n.source)
 
-    def add_atom(self, identifier, probability, group=None, name=None, source=None):
+    def add_atom(self, identifier, probability, group=None, name=None, source=None, cr_extra=True):
         """Add an atom to the formula.
 
         :param identifier: a unique identifier for the atom
@@ -556,6 +594,7 @@ class LogicFormula(BaseFormula):
         :param group: a group identifier that identifies mutually exclusive atoms (or None if no \
         constraint)
         :param name: name of the new node
+        :param cr_extra: When required, create an extra_node for the constraint group.
         :returns: the identifiers of the node in the formula (returns self.TRUE for deterministic \
         atoms)
 
@@ -580,16 +619,17 @@ class LogicFormula(BaseFormula):
             return self.TRUE
         else:
             atom = self._create_atom(identifier, probability, group, name, source)
+            length_before = len(self._nodes)
             node_id = self._add(atom, key=identifier)
 
             self.get_weights()[node_id] = probability
             if name is not None:
                 self.add_name(name, node_id, self.LABEL_NAMED)
-            if node_id == len(self._nodes):
+            if len(self._nodes) != length_before:
                 # The node was not reused?
                 self._atomcount += 1
                 # TODO if the next call return 0 or None, the node is still added?
-                node_id = self._add_constraint_me(group, node_id)
+                node_id = self._add_constraint_me(group, node_id, cr_extra=cr_extra)
             return node_id
 
     def add_and(self, components, key=None, name=None, compact=None):
@@ -715,7 +755,7 @@ class LogicFormula(BaseFormula):
 
             # If node has only one child, just return the child.
             # Don't do this for modifiable nodes, we need to keep a separate node.
-            if (readonly and update is None) and len(content) == 1:
+            if readonly and update is None and len(content) == 1:
                 if self._avoid_name_clash:
                     name_old = self.get_node(abs(content[0])).name
                     if name is None or name_old is None or name == name_old:
@@ -761,6 +801,14 @@ class LogicFormula(BaseFormula):
         for i, n in enumerate(self._nodes):
             yield (i + 1, n, type(n).__name__)
 
+    def __reversed__(self):
+        """Iterate over the nodes in the formula in the opposite direction.
+
+        :returns: iterator over tuples ( key, node, type )
+        """
+        for i, n in reversed(self._nodes):
+            yield (i + 1, n, type(n).__name__)
+
     def __len__(self):
         """Returns the number of nodes in the formula."""
         return len(self._nodes)
@@ -1017,7 +1065,7 @@ class LogicFormula(BaseFormula):
         .. code-block:: python
 
             pl = problog.program.PrologFile(input_file)
-            problog.formula.LogicFormula.create_from(avoid_name_clash=True, keep_order=True, \
+            problog.formula.LogicFormula.create_from(pl, avoid_name_clash=True, keep_order=True, \
 label_all=True)
             prologfile = gp.to_prolog()
 
@@ -1205,6 +1253,11 @@ label_all=True)
                         choice_name[p] = n.name
 
         for group, choices in choice_by_group.items():
+            # Check for single atom AD.
+            if len(choices) == 1 and choice_name.get(choices[0]) is None:
+                processed[choices[0]] = False  # Revert to not yet processed.
+                continue
+
             # Construct head
             head = Or.from_list([choice_name[c].with_probability(choice_prob[c]) for c in choices])
 
@@ -1512,15 +1565,16 @@ label_all=True)
         return s + '}'
 
     def clone(self, destination):
+        destination._auto_compact = False
         source = self
         # TODO maintain a translation table
         for i, n, t in source:
             if t == 'atom':
-                j = destination.add_atom(n.identifier, n.probability, n.group, source.get_name(i))
+                j = destination.add_atom(n.identifier, n.probability, n.group, name=source.get_name(i))
             elif t == 'conj':
-                j = destination.add_and(n.children, source.get_name(i))
+                j = destination.add_and(n.children, name=n.name)
             elif t == 'disj':
-                j = destination.add_or(n.children, source.get_name(i))
+                j = destination.add_or(n.children, name=n.name)
             else:
                 raise TypeError('Unknown node type')
             assert i == j
