@@ -79,7 +79,7 @@ This module contains basic logic constructs.
 """
 from __future__ import print_function
 from __future__ import division  # consistent behaviour of / and // in python 2 and 3
-
+import functools
 import math
 import sys
 import re
@@ -203,6 +203,8 @@ class Term(object):
         self._cache_is_ground = None
         self._cache_list_length = None
         self._cache_variables = None
+        self.repr = None
+        self.reprhash = None
 
     @property
     def functor(self):
@@ -349,6 +351,9 @@ class Term(object):
                     return new_term
 
     def __repr__(self):
+        if self.repr is not None:
+            return self.repr
+
         # Non-recursive version of __repr__
         stack = [deque([self])]
         # current: popleft from stack[-1]
@@ -476,7 +481,9 @@ class Term(object):
                 put(str(current))
             while stack and not stack[-1]:
                 stack.pop(-1)
-        return ''.join(parts)
+        self.repr = ''.join(parts)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     def __call__(self, *args, **kwdargs):
         """Create a new Term with the same functor and the given arguments.
@@ -859,9 +866,11 @@ class Clause(Term):
 
     def __repr__(self):
         if self.head.functor == '_directive':
-            return ":- %s" % self.body
+            self.repr = ":- %s" % self.body
         else:
-            return "%s :- %s" % (self.head, self.body)
+            self.repr = "%s :- %s" % (self.head, self.body)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     @property
     def predicates(self):
@@ -878,9 +887,11 @@ class AnnotatedDisjunction(Term):
 
     def __repr__(self):
         if self.body is None:
-            return "%s" % ('; '.join(map(str, self.heads)))
+            self.repr = "%s" % ('; '.join(map(str, self.heads)))
         else:
-            return "%s :- %s" % ('; '.join(map(str, self.heads)), self.body)
+            self.repr = "%s :- %s" % ('; '.join(map(str, self.heads)), self.body)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     @property
     def predicates(self):
@@ -934,7 +945,9 @@ class Or(Term):
     def __repr__(self):
         lhs = term2str(self.op1)
         rhs = term2str(self.op2)
-        return "%s; %s" % (lhs, rhs)
+        self.repr = "%s; %s" % (lhs, rhs)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     def with_args(self, *args):
         return self.__class__(*args, location=self.location)
@@ -996,7 +1009,9 @@ class And(Term):
         if isinstance(self.op1, Or):
             lhs = '(%s)' % lhs
 
-        return "%s, %s" % (lhs, rhs)
+        self.repr = "%s, %s" % (lhs, rhs)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     def with_args(self, *args):
         return self.__class__(*args, location=self.location)
@@ -1014,9 +1029,11 @@ class Not(Term):
         if isinstance(self.child, And) or isinstance(self.child, Or):
             c = '(%s)' % c
         if self.functor == 'not':
-            return 'not %s' % c
+            self.repr = 'not %s' % c
         else:
-            return '%s%s' % (self.functor, c)
+            self.repr = '%s%s' % (self.functor, c)
+        self.reprhash = hash(self.repr)
+        return self.repr
 
     def is_negated(self):
         return True
@@ -1148,4 +1165,4 @@ class InstantiationError(GroundingError):
 # noinspection PyShadowingBuiltins
 class ArithmeticError(GroundingError):
     """Error used when an error occurs during evaluation of an arithmetic expression."""
-    pass
+    pass
\ No newline at end of file
