@@ -149,10 +149,11 @@ SPECIAL_STRING = 10
 SPECIAL_ARGLIST = 11
 SPECIAL_SHARP_OPEN = 12
 SPECIAL_SHARP_CLOSE = 13
+SPECIAL_HEX_INTEGER = 14
 
 import re
 
-RE_FLOAT = re.compile(r'[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?')
+RE_FLOAT = re.compile(r'(0x[0-9a-fA-F]+)|([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)')
 
 
 def skip_to(s, pos, char):
@@ -175,11 +176,11 @@ def skip_comment_line(s, pos):
 
 
 def is_lower(c):
-    return 'a' <= c <= 'z'
+    return c.islower() #'a' <= c <= 'z'
 
 
 def is_upper(c):
-    return 'A' <= c <= 'Z'
+    return c.isupper() #'A' <= c <= 'Z'
 
 
 def is_digit(c):
@@ -456,7 +457,7 @@ class PrologParser(object):
             return Token('~>', pos, binop=(700, 'xfx', self.factory.build_binop),
                          functor=self._next_paren_open(s, pos)), pos + 2
         elif s[pos:pos + 2] == '~=':
-            return Token('~=', pos, unop=(200, 'fy', self.factory.build_unop),
+            return Token('~=', pos, binop=(700, 'xfx', self.factory.build_binop), unop=(200, 'fy', self.factory.build_unop),
                          functor=self._next_paren_open(s, pos)), pos + 2
         else:
             return Token('~', pos, unop=(900, 'fx', self.factory.build_unop),
@@ -492,7 +493,9 @@ class PrologParser(object):
 
     def _token_number(self, s, pos):
         token = RE_FLOAT.match(s, pos).group(0)
-        if token.find('.') >= 0 or token.find('e') >= 0 or token.find('E') >= 0:
+        if token.startswith('0x'):
+            return Token(token, pos, special=SPECIAL_HEX_INTEGER), pos + len(token)
+        elif token.find('.') >= 0 or token.find('e') >= 0 or token.find('E') >= 0:
             return Token(token, pos, special=SPECIAL_FLOAT), pos + len(token)
         else:
             return Token(token, pos, special=SPECIAL_INTEGER), pos + len(token)
@@ -515,6 +518,10 @@ class PrologParser(object):
             return self._token_lower
         elif c < 127:
             return self._token_act4[c - 123]
+        elif char.isalpha() and char.islower():
+            return self._token_lower
+        elif char.isalpha() and char.isupper():
+            return self._token_upper
         else:
             return None
 
@@ -631,6 +638,8 @@ class PrologParser(object):
                 return self.factory.build_variable(token.string, location=token.location)
             elif token.is_special(SPECIAL_INTEGER):
                 return self.factory.build_constant(int(token.string), location=token.location)
+            elif token.is_special(SPECIAL_HEX_INTEGER):
+                return self.factory.build_constant(int(token.string, 16), location=token.location)
             elif token.is_special(SPECIAL_FLOAT):
                 return self.factory.build_constant(float(token.string), location=token.location)
             elif token.is_special(SPECIAL_STRING):
